%module Minise
%{
#include "include/minise.hpp"
%}
%rename(Result) SE::SeResult;

%include "std_string.i"
%include "std_vector.i"
%template(Results) std::vector<SE::SeResult>;

%apply int { uint32_t };
#ifdef SWIGPERL
%typemap(in) (std::vector<uint32_t> &)(std::vector<uint32_t> vec){
    if (! SvROK ($input) or SvTYPE (SvRV ($input)) != SVt_PVAV)
	croak ("not a list");
    int len = av_len (reinterpret_cast <AV *> (SvRV ($input))) + 1;
    vec.reserve (len);
    for (int i = 0; i < len; ++i) {
	SV **po = av_fetch (reinterpret_cast <AV *> (SvRV ($input)), i, 0);
	if (! SvUOK (*po))
	    croak ("not an integer");
	vec.push_back (SvUV (*po));
    }
    $1 = &vec;
}
%typemap(out) (std::vector<uint32_t> *) (std::vector<uint32_t> &){
    int len = $1->size();
    AV *av = (AV *)sv_2mortal((SV *)newAV());
    av_extend(av, len);
    for(int i = 0; i < len; i++){
	av_push(av, newSViv((int)$1->at(i)));
    }
    $result = newRV_noinc((SV *)av);
    argvi++;
}
%typemap(out) std::vector<uint32_t> {
    int len = $1.size();
    AV *av = (AV *)sv_2mortal((SV *)newAV());
    av_extend(av, len);
    for(int i = 0; i < len; i++){
	av_push(av, newSViv((int)$1[i]));
    }
    $result = av;
    argvi++;
}
#endif

%include "exception.i"
%exception {
    try {
	$action
    } catch (const std::exception& e) {
	SWIG_exception(SWIG_RuntimeError, e.what());
    } catch (char *e) {
	SWIG_exception (SWIG_RuntimeError, e);
    } catch (const char *e) {
	SWIG_exception (SWIG_RuntimeError, (char*)e);
    }
}

%extend SE::Minise {
    std::string getSnippet(const uint32_t docID, const int offset, const uint32_t len){
	std::string ret;
	self->getSnippet(docID, offset, len, ret);
	return ret;
    }
}

%{
SE::Minise* openIndexFile(const char *index){
    using namespace SE;
    Minise::IndexType indexType = Minise::QUICKSEARCH;
    if(getIndexType(index, indexType) == -1){
	throw "openIndexFile read error";
    }

    Minise* ms = NULL;
    if (indexType == Minise::QUICKSEARCH){
	ms = new SE::QuickSearch;
    } else if (indexType == Minise::ONEGRAM ||
	       indexType == Minise::TWOGRAM ||
	       indexType == Minise::INVERTEDFILE){
	ms = new InvertedFile;
    } else if (indexType == Minise::SUFFIXARRAY ||
	       indexType == Minise::SUFFIXARRAY_UTF8){
	ms = new SuffixArray;
    } else {
	throw "openIndexFile unknown indexType";
    }

    if (ms->load(index) == -1){
	throw ms->what();
    }
    return ms;
}
%}

%include "include/miniseBase.hpp"
%include "include/invertedFile.hpp"
%include "include/suffixArray.hpp"
%include "include/quickSearch.hpp"
SE::Minise* openIndexFile(const char *index);
