#!/usr/bin/perl

use lib './blib/lib';
use lib './blib/arch';
use strict;
use Minise;

my $file = $ARGV[0];

my $minise = Minise::openIndexFile($file);
print 'minise=', $minise, "\n";
print 'indexname=', $minise->getIndexName, "\n";

my $query = 'help';
my $snippet_length = 100;

# http://d.hatena.ne.jp/ny23/20110504/p1
# http://d.hatena.ne.jp/higepon/20060223/1140684810
# に既述されているような typemap の記述方法を使うと，以下のように
# array の reference を渡すことができる．
#
#     my @ret;
#     $minise->search( $query, length $query, \@ret );
#
# ただ，返り値をどうやって受け取るかが分からなかったので，とりあえず
# http://www.swig.org/Doc1.3/Library.html#Library_stl_cpp_library
# に解説されている std_vector.i を使う方法で定義．そうすると，以下のよ
# うな呼び出し方になる．

my $results = Minise::Results->new;
$minise->search( $query, length $query, $results );
print 'results=', $results, "\n";
print 'results->size=', $results->size, "\n";
for( my $i = 0; $i < $results->size; $i++ ){
    print "==== $i ====\n";
    my $res = $results->get($i);
    print 'res=', $res, "\n";
    print 'title=', $res->swig_title_get, "\n";
    print 'docID=', $res->swig_docID_get, "\n";
    my $offsets = $res->swig_offsets_get;
    for( my $j = 0; $j <= $#{$offsets}; $j++ ){
	print 'offset=', $offsets->[$j], "\n";
	print $minise->getSnippet($res->swig_docID_get, $offsets->[$j], $snippet_length), "\n";
    }
}
